from __future__ import unicode_literals
__all__ = (
    'ui_style',
)


ui_style = {
    # Standout is caused by man pages that insert a x\b in the output.
    'standout': 'bold #44aaff',
    'standout2': 'underline #aa8844',

    # Show current search term. (The cursor is not visible).
    'search': 'bg:default fg:default reverse',
    'search.current': 'bg:#aaff00 #000000 noreverse',

    # UI style
    'statusbar': 'reverse',
    'statusbar cursor-position': 'noreverse #ffffff bg:#884400',
    'statusbar key': 'bold',

    'arg': 'reverse #ccaa00',

    'search-toolbar': 'bg:#333333 #ffffff',
    'search-toolbar text': '#ffffff',
    'system-toolbar': 'bg:#333333 #ffffff',
    'system-toolbar text': '#ffffff',

    'examine': 'bg:#333333 #ffffff',
    'examine-text': 'bg:#aa88ff #000000',

    'titlebar': 'bg:#333333 #ffffff',

    # Messages
    'message':  'bg:#bbee88 #222222',

    'loading':  '#888888',

    # Help
    'title': 'bold #44aaff',
    'subtitle': 'bold #8800ff',
    'line': '#888888',
    'keys': '#44aaff',
    'version': '#8800ff',
}



from __future__ import unicode_literals
from prompt_toolkit.application import get_app
from prompt_toolkit.filters import has_focus, Condition
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.key_binding.bindings.scroll import scroll_page_up, scroll_page_down, scroll_one_line_down, scroll_one_line_up, scroll_half_page_up, scroll_half_page_down
from prompt_toolkit.keys import Keys
from prompt_toolkit.search import stop_search
from prompt_toolkit.utils import suspend_to_background_supported


__all__ = (
    'create_key_bindings',
)

def create_key_bindings(pager):
    kb = KeyBindings()
    handle = kb.add

    @Condition
    def has_colon():
        return pager.in_colon_mode

    @Condition
    def default_focus():
        app = get_app()
        return app.layout.current_window == pager.current_source_info.window

    @Condition
    def displaying_help():
       return pager.displaying_help

    for c in '01234556789':
        @handle(c, filter=default_focus)
        def _(event, c=c):
            event.append_to_arg_count(c)

    @handle('q', filter=default_focus)
    @handle('Q', filter=default_focus | has_colon)
    @handle('Z', 'Z', filter=default_focus)
    def _(event):
        " Quit. "
        if pager.displaying_help:
            pager.quit_help()
        else:
            event.app.exit()

    @handle(' ', filter=default_focus)
    @handle('f', filter=default_focus)
    @handle(Keys.ControlF, filter=default_focus)
    @handle(Keys.ControlV, filter=default_focus)
    def _(event):
        " Page down."
        scroll_page_down(event)

    @handle('b', filter=default_focus)
    @handle(Keys.ControlB, filter=default_focus)
    @handle(Keys.Escape, 'v', filter=default_focus)
    def _(event):
        " Page up."
        scroll_page_up(event)

    @handle('d', filter=default_focus)
    @handle(Keys.ControlD, filter=default_focus)
    def _(event):
        " Half page down."
        scroll_half_page_down(event)

    @handle('u', filter=default_focus)
    @handle(Keys.ControlU, filter=default_focus)
    def _(event):
        " Half page up."
        scroll_half_page_up(event)

    @handle('e', filter=default_focus)
    @handle('j', filter=default_focus)
    @handle(Keys.ControlE, filter=default_focus)
    @handle(Keys.ControlN, filter=default_focus)
    @handle(Keys.ControlJ, filter=default_focus)
    @handle(Keys.ControlM, filter=default_focus)
    @handle(Keys.Down, filter=default_focus)
    def _(event):
        " Scoll one line down."
        if event.arg > 1:
            # When an argument is given, go this amount of lines down.
            event.current_buffer.auto_down(count=event.arg)
        else:
            scroll_one_line_down(event)

    @handle('y', filter=default_focus)
    @handle('k', filter=default_focus)
    @handle(Keys.ControlY, filter=default_focus)
    @handle(Keys.ControlK, filter=default_focus)
    @handle(Keys.ControlP, filter=default_focus)
    @handle(Keys.Up, filter=default_focus)
    def _(event):
        " Scoll one line up."
        if event.arg > 1:
            event.current_buffer.auto_up(count=event.arg)
        else:
            scroll_one_line_up(event)

    @handle(Keys.Escape, 'u')
    def _(event):
        " Toggle search highlighting. "
        pager.highlight_search = not pager.highlight_search

    @handle('=', filter=default_focus)
    @handle(Keys.ControlG, filter=default_focus)
    @handle('f', filter=has_colon)
    def _(event):
        " Print the current file name. "
        pager.message = ' {} '.format(pager.current_source.get_name())

    @handle('h', filter=default_focus & ~displaying_help)
    @handle('H', filter=default_focus & ~displaying_help)
    def _(event):
        " Display Help. "
        pager.display_help()

    @handle('g', filter=default_focus)
    @handle('<', filter=default_focus)
    @handle(Keys.Escape, '<', filter=default_focus)
    def _(event):
        " Go to the first line of the file. "
        event.current_buffer.cursor_position = 0

    @handle('G', filter=default_focus)
    @handle('>', filter=default_focus)
    @handle(Keys.Escape, '>', filter=default_focus)
    def _(event):
        " Go to the last line of the file. "
        b = event.current_buffer
        b.cursor_position = len(b.text)

    @handle('m', Keys.Any, filter=default_focus)
    def _(event):
        " Mark current position. "
        source_info = pager.current_source_info

        source_info.marks[event.data] = (
            event.current_buffer.cursor_position,
            source_info.window.vertical_scroll)

    @handle("'", Keys.Any, filter=default_focus)
    def _(event):
        " Go to a previously marked position. "
        go_to_mark(event, event.data)

    @handle(Keys.ControlX, Keys.ControlX, filter=default_focus)
    def _(event):
        " Same as '. "
        go_to_mark(event, '.')

    def go_to_mark(event, mark):
        b = event.current_buffer
        source_info = pager.current_source_info
        try:
            if mark == '^':  # Start of file.
                cursor_pos, vertical_scroll = 0, 0
            elif mark == '$':  # End of file - mark.
                cursor_pos, vertical_scroll = len(b.text), 0
            else:  # Custom mark.
                cursor_pos, vertical_scroll = source_info.marks[mark]
        except KeyError:
            pass  # TODO: show warning.
        else:
            b.cursor_position = cursor_pos
            source_info.window.vertical_scroll = vertical_scroll

    @handle('F', filter=default_focus)
    def _(event):
        " Forward forever, like 'tail -f'. "
        pager.forward_forever = True

    @handle('r', filter=default_focus)
    @handle('R', filter=default_focus)
    def _(event):
        event.app.renderer.clear()

    def search_buffer_is_empty():
        " Returns True when the search buffer is empty. "
        return pager.search_buffer.text == ''

    @handle('backspace', filter=has_focus(pager.search_buffer) & Condition(search_buffer_is_empty))
    def _(event):
        " Cancel search when backspace is pressed. "
        stop_search()

    @handle(Keys.Left, filter=default_focus)
    @handle(Keys.Escape, '(', filter=default_focus)
    def _(event):
        " Scroll half page to the left. "
        w = event.app.layout.current_window
        b = event.app.current_buffer

        if w and w.render_info:
            info = w.render_info
            amount = info.window_width // 2

            # Move cursor horizontally.
            value = b.cursor_position - min(amount, len(b.document.current_line_before_cursor))
            b.cursor_position = value

            # Scroll.
            w.horizontal_scroll = max(0, w.horizontal_scroll - amount)

    @handle(Keys.Right, filter=default_focus)
    @handle(Keys.Escape, ')', filter=default_focus)
    def _(event):
        " Scroll half page to the right. "
        w = event.app.layout.current_window
        b = event.app.current_buffer

        if w and w.render_info:
            info = w.render_info
            amount = info.window_width // 2

            # Move the cursor first to a visible line that is long enough to
            # have the cursor visible after scrolling. (Otherwise, the Window
            # will scroll back.)
            xpos = w.horizontal_scroll + amount

            for line in info.displayed_lines:
                if len(b.document.lines[line]) >= xpos:
                    b.cursor_position = b.document.translate_row_col_to_index(line, xpos)
                    break

            # Scroll.
            w.horizontal_scroll = max(0, w.horizontal_scroll + amount)

    @handle(':', filter=default_focus & ~displaying_help)
    def _(event):
        pager.in_colon_mode = True

    @handle('n', filter=has_colon)
    def _(event):
        " Go to next file. "
        pager.focus_next_source()

    @handle('p', filter=has_colon)
    def _(event):
        " Go to previous file. "
        pager.focus_previous_source()

    @handle('e', filter=has_colon)
    @handle(Keys.ControlX, Keys.ControlV, filter=default_focus)
    def _(event):
        event.app.layout.focus(pager.layout.examine_control)
        pager.in_colon_mode = False

    @handle('d', filter=has_colon)
    def _(event):
        pager.remove_current_source()

    @handle('backspace', filter=has_colon)
    @handle('q', filter=has_colon)
    def _(event):
        pager.in_colon_mode = False

    @handle(Keys.Any, filter=has_colon)
    def _(event):
        pager.in_colon_mode = False
        pager.message = 'No command.'

    @handle(Keys.ControlC, filter=has_focus('EXAMINE'))
    @handle(Keys.ControlG, filter=has_focus('EXAMINE'))
    def _(event):
        " Cancel 'Examine' input. "
        event.app.layout.focus(pager.current_source_info.window)

    @handle(Keys.ControlZ, filter=Condition(lambda: suspend_to_background_supported()))
    def _(event):
        " Suspend to bakground. "
        event.app.suspend_to_background()

    return kb



from __future__ import unicode_literals
from prompt_toolkit.filters import Filter

__all__ = (
    'HasColon',
)

class _PagerFilter(Filter):
    def __init__(self, pager):
        self.pager = pager


class HasColon(_PagerFilter):
    """
    The user typed a ':'.
    """
    def __call__(self):
        return self.pager.in_colon_mode


class DisplayingHelp(_PagerFilter):
    def __call__(self):
        return self.pager.displaying_help



from __future__ import unicode_literals
from prompt_toolkit.application import get_app
from prompt_toolkit.enums import SYSTEM_BUFFER
from prompt_toolkit.filters import HasArg, Condition, HasSearch, has_focus
from prompt_toolkit.formatted_text import HTML
from prompt_toolkit.layout.containers import HSplit, VSplit, Window, ConditionalContainer, Float, FloatContainer, Container, WindowAlign
from prompt_toolkit.layout.controls import BufferControl, FormattedTextControl
from prompt_toolkit.layout.menus import MultiColumnCompletionsMenu
from prompt_toolkit.layout.processors import Processor, HighlightIncrementalSearchProcessor, HighlightMatchingBracketProcessor, TabsProcessor, Transformation, ConditionalProcessor, BeforeInput, HighlightSelectionProcessor, HighlightSearchProcessor
from prompt_toolkit.lexers import SimpleLexer
from prompt_toolkit.widgets.toolbars import SearchToolbar, SystemToolbar, FormattedTextToolbar

from .filters import HasColon

import weakref

__all__ = (
    'PagerLayout',
)


class _EscapeProcessor(Processor):
    """
    Interpret escape sequences like less/more/most do.
    """
    def __init__(self, source_info):
        self.source_info = source_info

    def apply_transformation(self, ti):
        tokens = self.source_info.line_tokens[ti.lineno]
        return Transformation(tokens[:])


class _Arg(ConditionalContainer):
    def __init__(self):
        def get_text():
            app = get_app()
            if app.key_processor.arg is not None:
                return  ' %s ' % app.key_processor.arg
            else:
                return ''

        super(_Arg, self).__init__(
                Window(FormattedTextControl(get_text), style='class:arg',
                       align=WindowAlign.RIGHT),
                filter=HasArg())


class Titlebar(FormattedTextToolbar):
    """
    Displayed at the top.
    """
    def __init__(self, pager):
        def get_tokens():
            return pager.titlebar_tokens

        super(Titlebar, self).__init__(get_tokens)


class MessageToolbarBar(FormattedTextToolbar):
    """
    Pop-up (at the bottom) for showing error/status messages.
    """
    def __init__(self, pager):
        def get_tokens():
            return [('class:message', pager.message)] if pager.message else []

        super(MessageToolbarBar, self).__init__(get_tokens)


class _DynamicBody(Container):
    def __init__(self, pager):
        self.pager = pager
        self._bodies = weakref.WeakKeyDictionary()  # Map buffer_name to Window.

    def get_buffer_window(self):
        " Return the Container object according to which Buffer/Source is visible. "
        return self.pager.current_source_info.window

    def reset(self):
        for body in self._bodies.values():
            body.reset()

    def get_render_info(self):
        return self.get_buffer_window().render_info

    def preferred_width(self, *a, **kw):
        return self.get_buffer_window().preferred_width(*a, **kw)

    def preferred_height(self, *a, **kw):
        return self.get_buffer_window().preferred_height(*a, **kw)

    def write_to_screen(self, *a, **kw):
        return self.get_buffer_window().write_to_screen(*a, **kw)

    def get_children(self):
        return [self.get_buffer_window()]

    def walk(self, *a, **kw):
        # Required for prompt_toolkit.layout.utils.find_window_for_buffer_name.
        return self.get_buffer_window().walk(*a, **kw)


class PagerLayout(object):
    def __init__(self, pager):
        self.pager = pager
        self.dynamic_body = _DynamicBody(pager)

        # Build an interface.
        has_colon = HasColon(pager)

        self.examine_control = BufferControl(
            buffer=pager.examine_buffer,
            lexer=SimpleLexer(style='class:examine,examine-text'),
            input_processors=[
                BeforeInput(lambda: [('class:examine', ' Examine: ')])
            ])

        self.search_toolbar = SearchToolbar(
            vi_mode=True,
            search_buffer=pager.search_buffer)

        self.container = FloatContainer(
            content=HSplit([
                ConditionalContainer(
                    content=Titlebar(pager),
                    filter=Condition(lambda: pager.display_titlebar)),
                self.dynamic_body,
                self.search_toolbar,
                SystemToolbar(),
                ConditionalContainer(
                    content=VSplit([
                            Window(height=1,
                                   content=FormattedTextControl(self._get_statusbar_left_tokens),
                                   style='class:statusbar'),
                            Window(height=1,
                                   content=FormattedTextControl(self._get_statusbar_right_tokens),
                                   style='class:statusbar.cursorposition',
                                   align=WindowAlign.RIGHT),
                        ]),
                    filter=~HasSearch() & ~has_focus(SYSTEM_BUFFER) & ~has_colon & ~has_focus('EXAMINE')),
                ConditionalContainer(
                    content=Window(
                        FormattedTextControl(' :'),
                        height=1,
                        style='class:examine'),
                    filter=has_colon),
                ConditionalContainer(
                    content=Window(
                        self.examine_control,
                        height=1,
                        style='class:examine'),
                    filter=has_focus(pager.examine_buffer)),
            ]),
            floats=[
                Float(right=0, height=1, bottom=1,
                      content=_Arg()),
                Float(bottom=1, left=0, right=0, height=1,
                      content=ConditionalContainer(
                          content=MessageToolbarBar(pager),
                          filter=Condition(lambda: bool(pager.message)))
                ),
                Float(right=0, height=1, bottom=1,
                      content=ConditionalContainer(
                          content=FormattedTextToolbar(
                              lambda: [('class:loading', ' Loading... ')],
                          ),
                          filter=Condition(lambda: pager.current_source_info.waiting_for_input_stream))),
                Float(xcursor=True,
                      ycursor=True,
                      content=MultiColumnCompletionsMenu()),
            ]
        )

    def _get_statusbar_left_tokens(self):
        """
        Displayed at the bottom left.
        """
        if self.pager.displaying_help:
            return HTML(' HELP -- Press <key>[q]</key> when done')
        else:
            return HTML(' (press <key>[h]</key> for help or <key>[q]</key> to quit)')

    def _get_statusbar_right_tokens(self):
        """
        Displayed at the bottom right.
        """
        buffer = self.pager.source_info[self.pager.current_source].buffer
        document = buffer.document
        row = document.cursor_position_row + 1
        col = document.cursor_position_col + 1

        if self.pager.current_source.eof():
            percentage = int(100 * row / document.line_count)
            return [
                    ('class:statusbar,cursor-position',
                 ' (%s,%s) %s%% ' % (row, col, percentage))]
        else:
            return [
                ('class:statusbar,cursor-position',
                 ' (%s,%s) ' % (row, col))]



def create_buffer_window(source_info):
    """
    Window for the main content.
    """
    pager = source_info.pager

    input_processors = [
        ConditionalProcessor(
            processor=_EscapeProcessor(source_info),
            filter=Condition(lambda: not bool(source_info.source.lexer)),
        ),
        TabsProcessor(),
        ConditionalProcessor(
            processor=HighlightSearchProcessor(),
            filter=Condition(lambda: pager.highlight_search),
        ),
        ConditionalProcessor(
            processor=HighlightIncrementalSearchProcessor(),
            filter=Condition(lambda: pager.highlight_search),
        ),
        HighlightSelectionProcessor(),
        HighlightMatchingBracketProcessor(),
    ]

    return Window(
        always_hide_cursor=True,
        content=BufferControl(
            buffer=source_info.buffer,
            lexer=source_info.source.lexer,
            input_processors=input_processors,
            include_default_input_processors=False,
            preview_search=True,
            search_buffer_control=pager.layout.search_toolbar.control))



#!/usr/bin/env python
"""
pypager: A pure Python pager application.
"""
from __future__ import unicode_literals
from prompt_toolkit.lexers import PygmentsLexer
from pypager.pager import Pager
from pypager.source import FileSource
from prompt_toolkit.utils import is_windows
from prompt_toolkit import __version__ as ptk_version

import argparse
import os
import sys

PTK3 = ptk_version.startswith('3.')

if not PTK3:
    from prompt_toolkit.eventloop.defaults import set_event_loop

__all__ = (
    'run',
)


def run():
    if not PTK3:
        if is_windows():
            from prompt_toolkit.eventloop.win32 import Win32EventLoop
            loop = Win32EventLoop()
        else:
            from prompt_toolkit.eventloop.posix import PosixEventLoop
            from prompt_toolkit.eventloop.select import SelectSelector
            loop = PosixEventLoop(selector=SelectSelector)
        set_event_loop(loop)

    if not sys.stdin.isatty():
        pager = Pager.from_pipe()
        pager.run()
    else:
        parser = argparse.ArgumentParser(description='Browse through a text file.')
        parser.add_argument('filename', metavar='filename', nargs='+',
                            help='The file to be displayed.')
        parser.add_argument('--vi', help='Prefer Vi key bindings.', action='store_true')
        parser.add_argument('--emacs', help='Prefer Emacs key bindings.', action='store_true')

        args = parser.parse_args()

        # Determine input mode.
        vi_mode = 'vi' in os.environ.get('EDITOR', '').lower()
        if args.vi: vi_mode = True
        if args.emacs: vi_mode = False

        pager = Pager(vi_mode=vi_mode)

        # Open files.
        for filename in args.filename:
            # When a filename is given, take a lexer from that filename.
            lexer = PygmentsLexer.from_filename(filename, sync_from_start=False)

            pager.add_source(FileSource(filename, lexer=lexer))

        # Run UI.
        pager.run()


if __name__ == '__main__':
    run()



#!/usr/bin/env python
import os
from setuptools import setup, find_packages
import pypager

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as f:
    long_description = f.read()

install_requires = [
    'prompt_toolkit>=2.0.0,<3.1.0',
    'pygments',
]

# Argparse became part of the standard library in Python 2.7
# Include for older versions.
try:
    import argparse
except ImportError:
    install_requires += ['argparse']

setup(
    name='pypager',
    author='Jonathan Slenders',
    version=pypager.__version__,
    license='LICENSE',
    url='https://github.com/jonathanslenders/pypager',
    description='Pure Python pager (like "more" and "less").',
    long_description=long_description,
    packages=find_packages('.'),
    install_requires=install_requires,
    entry_points={
        'console_scripts': [
            'pypager = pypager.entry_points.run_pypager:run',
        ]
    },
)
